unit Main;

interface

uses
  SysUtils, Windows, Messages, Graphics, Forms, IniFiles, Math, Classes,
  Dialogs, ExtDlgs, StdCtrls, Controls, Menus, PlotterUnit;

const
  MaxAfterDot = 15; // digits in hint

type
  TfrmMain = class(TForm)
    lblHint: TLabel;
    mnuPopup: TPopupMenu;
    mnuSaveFigure: TMenuItem;
    dlgSave: TSavePictureDialog;
    mnuSaveAllFigures: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure mnuSaveFigureClick(Sender: TObject);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure lblHintMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure mnuPopupPopup(Sender: TObject);
    procedure mnuSaveAllFiguresClick(Sender: TObject);
  private
    Bmp: TBitmap;
    FigureWindow: TFigureWindow;
    Xpop, Ypop: integer;
    HintType, AfterDot: integer; // for hint

    procedure Refresh; inline;
    procedure MakeHint(const X, Y: integer);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.Refresh;
begin
  BitBlt(
    Canvas.Handle, 0, 0, ClientWidth, ClientHeight,
    Bmp.Canvas.Handle, 0, 0, SRCCOPY);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  DoubleBuffered := true;

  FigureWindow := TFigureWindow.Create(OptionsFile);
  FigureWindow.Load;

  ClientWidth := FigureWindow.Width;
  ClientHeight := FigureWindow.Height;
  Constraints.MinWidth := FigureConstraint*FigureWindow.WFig +
    Width - ClientWidth;
  Constraints.MinHeight := FigureConstraint*FigureWindow.HFig +
    Height - ClientHeight;

  if FigureWindow.Caption <> '' then begin
    Caption := FigureWindow.Caption;
    Application.Title := FigureWindow.Caption;
  end;

  if PlottoFile <> '' then
    FigureWindow.SaveToFile(PlottoFile)
  else begin
    dlgSave.InitialDir := ExtractFilePath(Application.ExeName);
    dlgSave.FilterIndex := 1;
    dlgSave.DefaultExt := 'bmp';

    AfterDot := 4;

    Bmp := TBitmap.Create;
    Bmp.Width := ClientWidth;
    Bmp.Height := ClientHeight;
    Bmp.Canvas.Brush.Color := clBtnFace;
    Bmp.Canvas.FillRect(frmMain.ClientRect);

    FigureWindow.Canvas := Bmp.Canvas;
    FigureWindow.Draw;
  end;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if Bmp <> nil then Bmp.Free;
  if FigureWindow <> nil then FigureWindow.Free;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  if PlottoFile <> '' then Close;
end;

procedure TfrmMain.FormPaint(Sender: TObject);
begin
  Refresh;
end;

procedure TfrmMain.FormResize(Sender: TObject);
var
  k: integer;
begin
  k := frmMain.ClientWidth mod FigureWindow.WFig;
  frmMain.Width := frmMain.Width - k;
  k := frmMain.ClientHeight mod FigureWindow.HFig;
  frmMain.Height := frmMain.Height - k;
  if Bmp <> nil then begin
    Bmp.Width := ClientWidth;
    Bmp.Height := ClientHeight;
    Bmp.Canvas.Brush.Color := clBtnFace;
    Bmp.Canvas.FillRect(frmMain.ClientRect);

    FigureWindow.Resize(ClientWidth, ClientHeight);
    FigureWindow.Draw;

    Refresh;
  end;
end;

procedure TfrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  NeedUpdate: boolean;
begin
  NeedUpdate := false;
  case key of
    VK_ESCAPE: Close;
    VK_UP: if ssCtrl in Shift then NeedUpdate := FigureWindow.LessFigure(false);
    VK_DOWN: if ssCtrl in Shift then begin
      FigureWindow.MoreFigure(false);
      NeedUpdate := true;
    end;
    VK_LEFT:
      if ssCtrl in Shift then
        NeedUpdate := FigureWindow.LessFigure(true)
      else
        NeedUpdate := FigureWindow.PrevFigure;
    VK_RIGHT:
      if ssCtrl in Shift then begin
        FigureWindow.MoreFigure(true);
        NeedUpdate := true;
      end else
        NeedUpdate := FigureWindow.NextFigure;
    VK_PRIOR: NeedUpdate := FigureWindow.PrevFigure(false);
    VK_NEXT: NeedUpdate := FigureWindow.NextFigure(false);
    VK_HOME: begin
      FigureWindow.HomeFigure;
      NeedUpdate := true;
    end;
    VK_END: begin
      FigureWindow.EndFigure;
      NeedUpdate := true;
    end;
  end;
  if NeedUpdate then begin
    Constraints.MinWidth := FigureConstraint*FigureWindow.WFig +
      Width - ClientWidth;
    Constraints.MinHeight := FigureConstraint*FigureWindow.HFig +
      Height - ClientHeight;
    FormResize(Self);
  end;
end;

procedure TfrmMain.MakeHint(const X, Y: integer);
begin
  lblHint.Caption := FigureWindow.Hint(X, Y, AfterDot, HintType);
  if X + lblHint.Width > frmMain.ClientWidth then
    lblHint.Left := X - lblHint.Width
  else
    lblHint.Left := X;
  if Y + lblHint.Height > frmMain.ClientHeight then
    lblHint.Top := X - lblHint.Height
  else
    lblHint.Top := Y;
end;

procedure TfrmMain.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then begin
    MakeHint(X, Y);
    lblHint.Visible := lblHint.Caption <> '';
  end;
  Xpop := X; Ypop := Y;
end;

procedure TfrmMain.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  lblHint.Visible := false;
end;

procedure TfrmMain.FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
var
  Key: word;
begin
  if lblHint.Visible then begin
    if HintType = 1 then exit;
    Dec(AfterDot);
    if AfterDot < 0 then AfterDot := 0;
    MakeHint(Xpop, Ypop);
  end else if [ssLeft, ssMiddle, ssRight]*Shift = [] then begin
    if ssCtrl in Shift then Key := VK_NEXT
    else Key := VK_RIGHT;
    frmMain.KeyDown(Key, Shift);
  end;
end;

procedure TfrmMain.FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
var
  Key: word;
begin
  if lblHint.Visible then begin
    if HintType = 1 then exit;
    Inc(AfterDot);
    if AfterDot > MaxAfterDot then AfterDot := MaxAfterDot;
    MakeHint(Xpop, Ypop);
  end else if [ssLeft, ssMiddle, ssRight]*Shift = [] then begin
    if ssCtrl in Shift then Key := VK_PRIOR
    else Key := VK_LEFT;
    frmMain.KeyDown(Key, Shift);
  end;
end;

procedure TfrmMain.lblHintMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  lblHint.Visible := false;
end;

procedure TfrmMain.mnuPopupPopup(Sender: TObject);
begin
  lblHint.Visible := false;
end;

procedure TfrmMain.mnuSaveFigureClick(Sender: TObject);
var
  j, w, h: integer;
  r: TRect;
begin
  with FigureWindow do begin
    j := WhatFigure(Xpop, Ypop);
    if j < 0 then exit;

    r := GetRect(j);
    w := r.Right - r.Left + 1;
    h := r.Bottom - r.Top + 1;

    dlgSave.Title := 'Save figure';
    dlgSave.FileName := '';
    if dlgSave.Execute then
      Figures[j].SaveToFile(dlgSave.FileName, w, h);
  end;
end;

procedure TfrmMain.mnuSaveAllFiguresClick(Sender: TObject);
begin
  dlgSave.Title := 'Save figures';
  dlgSave.FileName := '';
  if dlgSave.Execute then
    FigureWindow.SaveToFile(dlgSave.FileName);
end;

end.
