/// Problem  : Hills (School Territory Olympiad 2006-2007)
/// Solution : Igor V. Burago

#include <fstream>
#include <iomanip>
#include <cassert>
#include <cmath>
using namespace std;

ifstream cin("input.txt");
ofstream cout("output.txt");

#define N_MAX 301
#define M_MAX 301
#define INF ((real)1e300)
#define EPS ((real)1e-10)
#define SQR(x) ((x)*(x))

typedef double real;

int n, m;
real H, A[N_MAX], B[N_MAX], C[N_MAX];
bool can[N_MAX][N_MAX];
real d[N_MAX][N_MAX], a[N_MAX][M_MAX];

inline int cmp(real a, real b)
{
  if (abs(a - b) < EPS) return 0;
  else if (a < b) return -1;
  else return 1;
}

inline real dist(real x1, real y1, real x2, real y2)
{
  return sqrt(SQR(x1 - x2) + SQR(y1 - y2));
}

void peak(int k, real &px, real &py)
{
  px = -0.5*B[k]/A[k];
  py = A[k]*px*px + B[k]*px + C[k] + H;
}

bool cross(int k, real x1, real y1, real x2, real y2)
{ // No vertical lines, i.e. x1 != x2
  real q, p, a, b, c, d;

  q = (y2 - y1)/(x2 - x1);
  p = (y1*x2 - x1*y2)/(x2 - x1);

  a = A[k];
  b = B[k] - q;
  c = C[k] - p;

  d = b*b - 4*a*c;
  if (cmp(d, 0) >= 0)
    for (int i = -1; i <= 1; i += 2)
    {
      real t, z, x = 0.5*(-b + i*sqrt(d))/a;
      t = (x - x1)/(x2 - x1);
      if (cmp(t, 0) >= 0 && cmp(t, 1) <= 0)
        return true;
    }

  return false;
}

int main()
{
  bool ok;
  int i, j, k;
  real x1, y1, x2, y2;

  cin >> n >> m >> H;
  assert(2 <= n && n <= 300);
  assert(2 <= m && m <= n);
  assert(1 <= H && H <= 1000000);
  for (i = 0; i < n; i++)
  {
    cin >> A[i] >> B[i] >> C[i];
    assert(-1000000 <= A[i] && A[i] < 0);
    assert(-1000000 <= B[i] && B[i] <= 1000000);
    assert(-1000000 <= C[i] && C[i] <= 1000000);
  }

  memset(can, 0, sizeof can);
  for (i = 0; i < n; i++)
    for (j = i+1; j < n; j++)
    {
      peak(i, x1, y1);
      peak(j, x2, y2);
      ok = true;
      for (k = 0; k < n; k++)
        if (cross(k, x1, y1, x2, y2))
        {
          ok = false;
          break;
        }
      if (can[i][j] = ok) // I mean assignment here
      {
        can[j][i] = can[i][j];
        d[i][j] = dist(x1, y1, x2, y2);
        d[j][i] = d[i][j];
      }
    }

  for (i = 0; i < n; i++)
    for (j = 0; j < m; j++)
      a[i][j] = INF;
  a[0][0] = 0; // It's compulsory to put one column at the first hill

  real best, num;
  for (i = 1; i < n; i++)
    for (j = 1; j < m; j++)
    {
      best = INF;
      for (k = 0; k < i; k++)
        if (can[k][i] && a[k][j-1] < INF)
        {
          num = a[k][j-1] + d[k][i];
          if (cmp(num, best) < 0)
            best = num;
        }
      a[i][j] = best;
    }

  if (a[n-1][m-1] < INF)
    cout << fixed << setprecision(5) << a[n-1][m-1];
  else
    cout << "-1";

  return 0;
}
