#include <cstdio>
#include <cstdlib>
#include <cstring>

#define MAX_N 301
#define INF ((double)1e300)
#define EVAL(x) (A*(x)*(x) + B*(x) + C)

int N, fmax = 0;
double H, A, B, C;
double DELTA = 5, STEP = 0.2;
char in_file[256];
bool show_cols = true, cool_min = false;

bool find_arg(int argc, char *argv[], char *arg)
{ 
  size_t x = strlen(arg);

  for (int i = 1; i < argc; i++)
    if (*argv[i] == '/' || *argv[i] == '-')
    {
      if (!strncmp((argv[i] + 1), arg, x) &&
          (argv[i][x + 1] == ':' || argv[i][x + 1] == '\0'))
        return true;
    }
    else
      break;

  return false;
}

char *arg_value(int argc, char *argv[], char *arg)
{
  size_t x = strlen(arg);

  for (int i = 1; i < argc; i++)
    if (*argv[i] == '/' || *argv[i] == '-')
    {
      if (!strncmp((argv[i] + 1), arg, x) && (argv[i][x + 1] == ':'))
        return (argv[i] + x + 2);
    }
    else
      break;

  return 0;
}

int main(int argc, char *argv[])
{
  if (argc > 1)
  {
    if (find_arg(argc, argv, "?"))
      printf("Usage: %s [-f:files_count] [-d:delta] [-s:step] [-m] [-c]\n", argv[0]);

    char *str;

    if (str = arg_value(argc, argv, "f"))
      sscanf(str, "%d", &fmax);
    if (str = arg_value(argc, argv, "d"))
      sscanf(str, "%lf", &DELTA);
    if (str = arg_value(argc, argv, "s"))
      sscanf(str, "%lf", &STEP);

    if (find_arg(argc, argv, "c"))
      show_cols = false;
    if (find_arg(argc, argv, "m"))
      cool_min = true;
  }

  freopen("plotter.ini", "wt", stdout);

  printf("[Window]\nBegFig = 1\nEndFig = %d\n", fmax);
  printf("[Defaults]\nPointSize = 0\nShowTitle = 1\n");
  if (show_cols)
    printf("clLine = $008800 $0000FF $000000\nLineWidth = 2 2 1\n");
  else
    printf("clLine = $008800\nLineWidth = 2\n");

  for (int j = 1; j <= fmax; j++)
  {
    sprintf(in_file, "%02d.in", j);

    freopen(in_file, "rt", stdin);    
    
    scanf("%d%*lf%lf", &N, &H);
    printf("[Figure%d]\nDataCount = %d\nTitle = %s\n", j, N + show_cols*(2*N-1), in_file);    

    int cnt = 0;
    double peak_x[MAX_N], peak_y[MAX_N];
    double mxmn = -INF;

    for (int i = 0; i < N; i++)
    {
      scanf("%lf%lf%lf", &A, &B, &C);

      double mn = INF, x, y, v = -0.5*B/A;

      peak_x[i] = v;
      peak_y[i] = EVAL(v);

      if (show_cols && i > 0)
      {
        printf("X%d = %0.8f %0.8f\n", cnt + 1, peak_x[i-1], peak_x[i]);
        printf("Y%d = %0.8f %0.8f\n", cnt + 1, peak_y[i-1] + H, peak_y[i] + H);
        cnt++;
      }

      printf("X%d = ", cnt + 1);
      for (x = v - DELTA; x <= v + DELTA; x += STEP)
        printf("%0.8f ", x);

      printf("\nY%d = ", cnt + 1);
      for (x = v - DELTA; x <= v + DELTA; x += STEP)
      {
        y = EVAL(x);
        if (y < mn) mn = y;
        printf("%0.8f ", y);
      }
      printf("\n");
      cnt++;

      if (show_cols)
      {
        printf("X%d = %0.8f %0.8f\n", cnt + 1, peak_x[i], peak_x[i]);
        printf("Y%d = %0.8f %0.8f\n", cnt + 1, peak_y[i], peak_y[i] + H);
        cnt++;
      }

      if (mn > mxmn) mxmn = mn;
    }

    if (cool_min)
      printf("MinY = %0.8f\n", mxmn);

    fclose(stdin);
  } 

  fclose(stdout);

  system("plotter.exe");

  return 0;
}
